import numpy as np
import collections

from math import *
from nllsCuda import pyPredict, pyQxy
from random import gauss, randint
import random

from Environment import Environment
from heuristics.AttractionAgent import *



GPUSTEPS = [256,256]

def calcPredictions(numTrials, samplePositions, env, gridSize):	
    global GPUSTEPS
    samples = []
    for trial in xrange(numTrials):
        t = []
        for i in xrange(len(samplePositions)):
            x,y = samplePositions[i]
            t.append((x,y,env.MeasuredPower(x,y)))
        samples.append(t)
    results = pyPredict(samples,GPUSTEPS,gridSize)

    return results

def simulate(numTrials, gridSize, emitterPosition, noiseStdDev, uavPositions, aiClass, heurParams):
    emitterPosition = list(emitterPosition)
    
    numReceivers = len(uavPositions)

    env = Environment(emitterPosition, 10, noiseStdDev)

    uavPositions = uavPositions[:]
    samplePositions = uavPositions[:]

    ai = aiClass(uavPositions,heurParams)
    
    while True: 
        
        results = calcPredictions(numTrials, samplePositions, env, gridSize)
        
        ai.update_state(results)
        for i,uavPosition in enumerate(uavPositions):
            oldPosition = uavPosition
            newPosition = ai.step(i)
            uavPositions[i] = newPosition
            samplePositions.append(oldPosition)
        
        
        
        predictions = results
        sum_x = sum(map(lambda x: x[0],predictions))
        sum_y = sum(map(lambda x: x[1],predictions))
        avg_x = round(sum_x / len(predictions),2)
        avg_y = round(sum_y / len(predictions),2)
        avg = (avg_x,avg_y)
        
        error_avg = round(distance(avg, emitterPosition),5)
        stddev = sum(map(lambda pos: ((pos[0]-avg_x)**2 +  (pos[0]-avg_x)**2), predictions))
        stddev /= len(predictions)
        
    
        if ((error_avg < 50.0 and stddev < 20.0) or len(samplePositions) > 100):
            return len(samplePositions)/len(uavPositions)
    
def make_receiver(base_position, angle, distance):
    r = (base_position[0]+distance*cos(angle),base_position[1]+distance*sin(angle))
    return r
    
def distance(pos1, pos2):
    return sqrt(sum(map(lambda x: (x[0]-x[1])**2, zip(pos1,pos2))))
        
def fitness(seed, numSimulations, numTrials, gridSize, emitterPosition, noiseStdDev, uavPositions, heurParams):
    numReceivers = len(uavPositions)
    results = []
    
    state = random.getstate()
    for s in xrange(numSimulations):
        random.seed(seed+s)
            
        samplesUsed = simulate(numTrials, gridSize, emitterPosition, noiseStdDev, uavPositions, AttractionAgent,heurParams)
        
        results.append(float(samplesUsed))
    random.setstate(state)
    
    results = np.array(results)
    avg = (results.sum()/len(results))
    stddev = sum(map(lambda v: v**2, results-avg))
    stddev /= len(results)
    stddev = sqrt(stddev)
    rmax = results.max()
    rmin = results.min()
    
    results = sorted(results)
    median = results[len(results)/2]
    
    return avg
        

